@@ -28,27 +28,57 @@ import os
 import re
 import subprocess
 import sys
+import six
+import codecs
 
 from . import config
 
 
+def open_text(filename, mode='r', encoding='utf-8'):
+    """An open() which removes some differences between Python 2 and 3 and
+    has saner defaults.
+
+    Unlike the builtin open by default utf-8 is use and not the locale
+    encoding (which is ANSI on Windows for example, not very helpful)
+
+    For Python 2, files are opened in text mode like with Python 3.
+    """
+
+    if mode not in ('r', 'w'):
+        raise ValueError("mode %r not supported, must be 'r' or 'w'" % mode)
+
+    if six.PY3:
+        return open(filename, mode, encoding=encoding)
+    else:
+        # We can't use io.open() here as its write method is too strict and
+        # only allows unicode instances and not everything in the codebase
+        # forces unicode at the moment. codecs.open() on the other hand
+        # happily takes ASCII str and decodes it.
+        return codecs.open(filename, mode, encoding=encoding)
+
+
 def setup_logging():
     """Check GTKDOC_TRACE environment variable.
 
     Set python log level to the value of the environment variable (DEBUG, INFO,
     WARNING, ERROR and CRITICAL) or INFO if the environment variable is empty.
     """
-    log_level = os.environ.get('GTKDOC_TRACE')
+    log_level = os.environ.get('GTKDOC_TRACE', 'WARNING')
     if log_level == '':
-        log_level = 'INFO'
-    if log_level:
-        logging.basicConfig(stream=sys.stdout,
-                            level=logging.getLevelName(log_level.upper()),
-                            format='%(asctime)s:%(filename)s:%(funcName)s:%(lineno)d:%(levelname)s:%(message)s')
-    # When redirecting the output on python2 we get UnicodeEncodeError:
-    if not sys.stdout.encoding:
-        import codecs
-        sys.stdout = codecs.getwriter('utf8')(sys.stdout)
+        log_level = 'WARNING'
+    logging.basicConfig(stream=sys.stdout,
+                        level=logging.getLevelName(log_level.upper()),
+                        format='%(asctime)s:%(filename)s:%(funcName)s:%(lineno)d:%(levelname)s:%(message)s')
+    # When redirecting the output on python2 or if run with a non utf-8 locale
+    # we get UnicodeEncodeError:
+    encoding = sys.stdout.encoding
+    if 'PYTHONIOENCODING' not in os.environ and (not encoding or encoding != 'UTF-8'):
+        sys.stdout.flush()
+        if six.PY3:
+            sys.stdout = open(sys.stdout.fileno(), mode='w', encoding='utf8', buffering=1)
+        else:
+            import codecs
+            sys.stdout = codecs.getwriter('utf8')(sys.stdout)
 
 
 def UpdateFileIfChanged(old_file, new_file, make_backup):
@@ -70,8 +100,11 @@ def UpdateFileIfChanged(old_file, new_file, make_backup):
     logging.debug("Comparing %s with %s...", old_file, new_file)
 
     if os.path.exists(old_file):
-        old_contents = open(old_file, 'rb').read()
-        new_contents = open(new_file, 'rb').read()
+        old_contents = new_contents = None
+        with open(old_file, 'rb') as f:
+            old_contents = f.read()
+        with open(new_file, 'rb') as f:
+            new_contents = f.read()
         if old_contents == new_contents:
             os.unlink(new_file)
             logging.debug("-> content is the same.")
@@ -99,9 +132,13 @@ def GetModuleDocDir(module_name):
       module_name (string): The module, e.g. 'glib-2.0'
 
     Returns:
-      str: the doc directory
+      str: the doc directory or None
     """
-    path = subprocess.check_output([config.pkg_config, '--variable=prefix', module_name], universal_newlines=True)
+    path = None
+    try:
+        path = subprocess.check_output([config.pkg_config, '--variable=prefix', module_name], universal_newlines=True)
+    except subprocess.CalledProcessError:
+        return None
     return os.path.join(path.strip(), 'share/gtk-doc/html')
 
 
@@ -116,7 +153,7 @@ def LogWarning(filename, line, message):
     filename = filename or "unknown"
 
     # TODO: write to stderr
-    print ("%s:%d: warning: %s" % (filename, line, message))
+    print("%s:%d: warning: %s" % (filename, line, message))
 
 
 def CreateValidSGMLID(xml_id):
